#!/usr/bin/env python

import os
import sys
import json
import datetime

from PyQt5.QtWidgets import QApplication, QMainWindow, QLabel
from PyQt5 import uic, QtWidgets
from PyQt5.QtWidgets import QFileDialog


class DataHolder(object):
    def __init__(self):
        self.dataHolderDomainName = "DomainName"
        self.dataHolderDomainCreationDate = "DomainCreationDate"


class DateTimeStamperMeta(type):
    __instance = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls.__instance:
            instance = super().__call__(*args, **kwargs)
            cls.__instance[cls] = instance
        return cls.__instance[cls]


class MyDateTimeStamper(metaclass=DateTimeStamperMeta):
    def __init__(self):
        self.__dateTimeStamp = None
        self.__dateTimeFormatting = "%Y-%m-%d_%H-%M-%S"

    def __generateDateTime(self):
        currentDateTime = datetime.datetime.now()
        self.__dateTimeStamp = currentDateTime.strftime(
            self.__dateTimeFormatting)

    def getDateTimeStamp(self):
        self.__generateDateTime()
        return self.__dateTimeStamp


class FileWriter(object):
    def __init__(self):
        self.__fileName = None
        self.__fileHandler = None
        self.__boolISWriteOk = False

    def openFile(self, fileName):
        self.__fileName = fileName
        try:
            self.__fileHandler = open(self.__fileName, "w")
        except OSError as exc:
            print("Exception Occured: {0}".format(exc))
            sys.exit(2)

    def getResultFileHandler(self):
        if self.__fileHandler is not None:
            return self.__fileHandler

    def writeFile(self, writeData):
        try:
            self.__fileHandler.write(writeData)
            self.__fileHandler.flush()
            self.__fileHandler.close()
            self.__boolISWriteOk = True
        except OSError as exc:
            print("Exception Occured: {0}".format(exc))
            sys.exit(3)

    def getResultWriteOK(self):
        if self.__boolISWriteOk == True:
            return True
        else:
            return False

    def __del__(self):
        if self.__fileHandler is not None:
            self.__fileHandler.close()


class PythonQtUIPostfixDomain(object):
    def __init__(self):
        self.__objFileWrite = FileWriter()
        self.__objDateTimeStamper = MyDateTimeStamper()
        self.__objDataHolder = DataHolder()
        self.__uiFileName = "./GUI_PyQt_Postfix_Domain-1.ui"
        self.__application = QApplication([])
        self.__mainWindow = QMainWindow()
        self.__UIC = uic.loadUi(self.__uiFileName, self.__mainWindow)
        self.__mainWindow.show()
        self.__buttonSaveToFile = None
        self.__labelTextOk = "Success"
        self.__labelTextFailed = "Failed"
        self.__outputFileHandler = None
        self.__lineEditValueDomain = None
        self.__dictJsonData = dict()
        self.__prefixDomainFileName = "PostfixDomain_"
        self.__postfixDomainFileName = ".json"
        self.__generatedDomainFullFileName = None
        self.__label3 = None

    def __statusLabel3Success(self):
        self.__UIC.label_3.setText(self.__labelTextOk)

    def __statusLabel3Failed(self):
        self.__UIC.label_3.setText(self.__labelTextFailed)

    def __readLineEdit(self):
        self.__lineEditValueDomain = self.__UIC.lineEdit.text()
        print("Domain Name Readed: {0}".format(self.__lineEditValueDomain))

    def __guiButton1(self):
        self.__buttonSaveToFile = self.__UIC.pushButton.clicked.connect(
            self.__actionSaveJsonToFile)

    def __actionSaveJsonToFile(self):
        self.__readLineEdit()
        self.__generateJSONDomainFileName()
        self.__saveToFileDialog()

    def __saveToFileDialog(self):
        messageDialogWindow = "Please Select Domain JSON Directory"
        selectedFileAndPathTuple = \
            QFileDialog.getSaveFileName(None,
                                        messageDialogWindow,
                                        self.__generatedDomainFullFileName,
                                        None)
        selectedFileAndPath = selectedFileAndPathTuple[0]
        print("After Dialog")
        print(selectedFileAndPath)
        if selectedFileAndPath != '':
            print(">>OK: Save to File.")
            self.__objFileWrite.openFile(selectedFileAndPath)
            self.__dumpJSONDomainName()
        else:
            print("Empty File Name Given in: GetSaveFileName")

    def __generateJSONDomainFileName(self):
        fileNameDomain = ""
        for char in self.__lineEditValueDomain:
            if char == ".":
                fileNameDomain += "_"
            else:
                fileNameDomain += char
        self.__generatedDomainFullFileName = self.__prefixDomainFileName + \
            fileNameDomain + \
            self.__postfixDomainFileName

    def __dumpJSONDomainName(self):
        self.__dictJsonData[self.__objDataHolder.dataHolderDomainName] = \
            self.__lineEditValueDomain
        self.__dictJsonData[
            self.__objDataHolder.dataHolderDomainCreationDate] = \
            self.__objDateTimeStamper.getDateTimeStamp()
        myDataJsonified = json.dumps(self.__dictJsonData, indent=1)
        self.__objFileWrite.writeFile(myDataJsonified)
        self.__statusLabel3Success()


    def run(self):
        self.__guiButton1()
        self.__application.exec_()


if __name__ == '__main__':
    pythonQtUIPostfixDomain = PythonQtUIPostfixDomain()
    pythonQtUIPostfixDomain.run()
